/*
 *
 * Copyright  2005, University of Washington, 
 * Department of Computer Science and Engineering. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer. 
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution.
 *
 * 3. Neither name of the University of Washington, Department of Computer 
 * Science and Engineering nor the names of its contributors may be used to 
 * endorse or promote products derived from this software without specific 
 * prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR 
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 *
 */

#ifndef FTDI466API_H
#define FTDI466API_H

#include "ftd2xx.h"
#include <windows.h>
#include <iostream>

using namespace std;

class FTDI466API
{
public:
	// constructor
	FTDI466API();

	// Loads the DLL, creates function pointers, and setups the FTDI chip for MPSSE
	// Returns: success
	bool open();

	// Closes the DLL, clears function pointers
	// Returns: success
	bool close(); 

	// Purges receive and transmit buffers in the device
	// Flags to pass in bufferMask: Any combination of FT_PURGE_RX and FT_PURGE_TX
	// Example: purgeBuffers(FT_PURGE_RX | FT_PURGE_TX);
	// Returns: success
	bool purgeBuffers(DWORD bufferMask); 

	// Gets the number of characters in the receive queue and the number of characters in the transmit queue
	// Parameters: 
	//		sizeRxQueue: pointer to a variable of type DWORD that will receive the number of characters in receive queue
	//		sizeTxQueue: pointer to a variable of type DWORD that will receive the number of characters in transmit queue
	// Returns success
	bool getQueueSizes(LPDWORD sizeRxQueue, LPDWORD sizeTxQueue); 

	// Returns the number of characters in the receive queue of the device
	DWORD getReceiveQueueSize();

	// Read data from the device
	// Parameters: 
	//		buffer: pointer to a buffer that receives the data
	//		numBytes: number of bytes to be read from the device
	// Returns: number of bytes read from the device
	DWORD read(LPVOID buffer, DWORD numBytes);

	// Write data to the device
	// Parameters: 
	//		buffer: pointer to a buffer that contains the data to be written to the device
	//		numBytes: number of bytes to write to the device
	// Returns: number of bytes written to the device
	DWORD write(LPVOID buffer ,DWORD numBytes);

private:
	// FTD2XX device functions
	FT_STATUS ListDevices(PVOID pvArg1, PVOID pvArg2, DWORD dwFlags);
	FT_STATUS OpenEx(PVOID pvArg1, DWORD dwFlags);	
	FT_STATUS Close();		
	FT_STATUS Read(LPVOID lpBuffer, DWORD dwBytesToRead, LPDWORD lpdwBytesReturned);
	FT_STATUS Write(LPVOID lpBuffer, DWORD dwBytesToWrite, LPDWORD lpdwBytesWritten);	
	FT_STATUS ResetDevice();
	FT_STATUS Purge(DWORD dwMask);
	FT_STATUS SetTimeouts(DWORD dwReadTimeout, DWORD dwWriteTimeout);
	FT_STATUS GetQueueStatus(LPDWORD lpdwAmountInRxQueue);
	FT_STATUS GetStatus(LPDWORD lpdwAmountInRxQueue, LPDWORD lpdwAmountInTxQueue, LPDWORD lpdwEventStatus);
	FT_STATUS SetLatencyTimer(UCHAR ucTimer);
	FT_STATUS GetBitMode(PUCHAR pucMode);	
	FT_STATUS SetBitMode(UCHAR ucMask, UCHAR ucEnable);

	// helper functions
	bool loadDLL();
	bool unloadDLL();
	bool syncMPSSE();
	void printError(DWORD error);

	// declare function pointer types
	typedef FT_STATUS (WINAPI *FnPtr_FT_ListDevices)(PVOID pvArg1, PVOID pvArg2, DWORD dwFlags);
	// small change
	typedef FT_STATUS (WINAPI *FnPtr_FT_OpenEx)(PVOID pvArg1, DWORD dwFlags, FT_HANDLE *ftHandle); // * didn't used to exist
	typedef FT_STATUS (WINAPI *FnPtr_FT_Close)(FT_HANDLE ftHandle);
	typedef FT_STATUS (WINAPI *FnPtr_FT_Read)(FT_HANDLE ftHandle, LPVOID lpBuffer, DWORD dwBytesToRead, LPDWORD lpdwBytesReturned);
	typedef FT_STATUS (WINAPI *FnPtr_FT_Write)(FT_HANDLE ftHandle, LPVOID lpBuffer, DWORD dwBytesToWrite, LPDWORD lpdwBytesWritten);
	typedef FT_STATUS (WINAPI *FnPtr_FT_ResetDevice)(FT_HANDLE ftHandle);
	typedef FT_STATUS (WINAPI *FnPtr_FT_Purge)(FT_HANDLE ftHandle, DWORD dwMask);
	typedef FT_STATUS (WINAPI *FnPtr_FT_SetTimeouts)(FT_HANDLE ftHandle, DWORD dwReadTimeout, DWORD dwWriteTimeout);
	typedef FT_STATUS (WINAPI *FnPtr_FT_GetQueueStatus)(FT_HANDLE ftHandle, LPDWORD lpdwAmountInRxQueue);
	typedef FT_STATUS (WINAPI *FnPtr_FT_GetStatus)(FT_HANDLE ftHandle, LPDWORD lpdwAmountInRxQueue, LPDWORD lpdwAmountInTxQueue, LPDWORD lpdwEventStatus);
	typedef FT_STATUS (WINAPI *FnPtr_FT_SetLatencyTimer)(FT_HANDLE ftHandle, UCHAR ucTimer);
	typedef FT_STATUS (WINAPI *FnPtr_FT_GetBitMode)(FT_HANDLE ftHandle, PUCHAR pucMode);
	typedef FT_STATUS (WINAPI *FnPtr_FT_SetBitMode)(FT_HANDLE ftHandle, UCHAR ucMask, UCHAR ucEnable);

	// handles
	HMODULE m_hFtdiDLL;
	FT_HANDLE m_hFtdiDevice;

	// function pointers
	FnPtr_FT_ListDevices m_pFnListDevices; 
	FnPtr_FT_OpenEx m_pFnOpenEx;
	FnPtr_FT_Close m_pFnClose;
	FnPtr_FT_Read m_pFnRead;
	FnPtr_FT_Write m_pFnWrite;
	FnPtr_FT_ResetDevice m_pFnResetDevice;
	FnPtr_FT_Purge m_pFnPurge;
	FnPtr_FT_SetTimeouts m_pFnSetTimeouts;
	FnPtr_FT_GetQueueStatus m_pFnGetQueueStatus;
	FnPtr_FT_GetStatus m_pFnGetStatus;
	FnPtr_FT_SetLatencyTimer m_pFnSetLatencyTimer;
	FnPtr_FT_GetBitMode m_pFnGetBitMode;
	FnPtr_FT_SetBitMode m_pFnSetBitMode;
	
};

#endif 